#!/usr/bin/python3
import os

from constants import Y_HEX_ACCENT1, Y_HEX_ACCENT2, Y_HEX_ACCENT3, Y_HEX_ACCENT4
from constants import y_hex_colors1, y_hex_colors2, y_hex_colors3, y_hex_colors4

def change_value (key, value, file):
    if value is not None:
        command = "sed -i '/%(key)s=/c\%(key)s=%(value)s' %(file)s" % {'key':key, 'value':value, 'file':file}
    else:
        command = "sed -i '/%(key)s=/d' %(file)s" % {'key':key, 'file':file}
    os.system(command)

def y_colorize_directory (path, variation):
    for accent in Y_HEX_ACCENT1:
        os.system("find %s -name '*.*' -type f -exec sed -i 's/%s/%s/gI' {}  \\;" % (path, accent, y_hex_colors1[variation]))
    for accent in Y_HEX_ACCENT2:
        os.system("find %s -name '*.*' -type f -exec sed -i 's/%s/%s/gI' {}  \\;" % (path, accent, y_hex_colors2[variation]))
    for accent in Y_HEX_ACCENT3:
        os.system("find %s -name '*.*' -type f -exec sed -i 's/%s/%s/gI' {}  \\;" % (path, accent, y_hex_colors3[variation]))
    for accent in Y_HEX_ACCENT4:
        os.system("find %s -name '*.*' -type f -exec sed -i 's/%s/%s/gI' {}  \\;" % (path, accent, y_hex_colors4[variation]))

if os.path.exists("usr"):
    os.system("rm -rf usr/")

os.system("mkdir -p usr/share/themes")

curdir = os.getcwd()

os.chdir("src/Mint-L")
os.system("./build-themes.py")
os.chdir(curdir)

# Mint-L color variations
for color in y_hex_colors1.keys():
    for variant in ["", "-Dark", "-Darker"]:
        original_name = "Mint-L%s" % variant
        path = os.path.join("src/Mint-L/variations/%s" % color)
        if os.path.isdir(path):
            print("Derivating %s-%s" % (original_name, color))

            # Copy theme
            theme = "usr/share/themes/%s-%s" % (original_name, color)
            theme_index = os.path.join(theme, "index.theme")
            os.system("cp -R usr/share/themes/%s %s" % (original_name, theme))

            # Theme name
            for key in ["Name", "GtkTheme"]:
                change_value(key, "%s-%s" % (original_name, color), theme_index)

            for key in ["IconTheme"]:
                change_value(key, "Mint-L-%s" % color, theme_index)

            for key in ["MetacityTheme"]:
                metacity_variant = original_name.replace("Darker", "Dark")
                change_value(key, "%s-%s" % (metacity_variant, color), theme_index)

            # Regenerate GTK3 sass
            os.system("cp -R src/Mint-L/gtk-3.0/sass %s/gtk-3.0/" % theme)
            y_colorize_directory("%s/gtk-3.0/sass" % theme, color)
            os.chdir("%s/gtk-3.0" % theme)
            if (variant == "-Dark"):
                os.system("cp sass/gtk-dark.scss sass/gtk.scss")
                os.system("sassc ./sass/gtk.scss gtk.css")
                os.system("sassc ./sass/gtk-dark.scss gtk-dark.css")
            elif (variant == "-Darker"):
                os.system("cp sass/gtk-darker.scss sass/gtk.scss")
                os.system("sassc ./sass/gtk.scss gtk.css")
                os.system("sassc ./sass/gtk-dark.scss gtk-dark.css")
            else:
                os.system("rm sass/gtk-dark.scss sass/gtk-darker.scss")
                os.system("sassc ./sass/gtk.scss gtk.css")

            os.system("rm -rf sass .sass-cache")
            os.chdir(curdir)

            # Regenerate Cinnamon sass
            if (variant != "-Darker"):
                # Darker variants have no cinnamon style
                os.system("cp -R src/Mint-L/cinnamon/sass %s/cinnamon/" % theme)
                y_colorize_directory("%s/cinnamon/sass" % theme, color)
                os.chdir("%s/cinnamon" % theme)
                if (variant == "-Dark"):
                    os.system("cp sass/cinnamon-dark.scss sass/cinnamon.scss")
                os.system("sassc ./sass/cinnamon.scss cinnamon.css")
                os.system("rm -rf sass .sass-cache")
                os.chdir(curdir)

            # Accent color
            files = []
            files.append(os.path.join(theme, "gtk-2.0", "gtkrc"))
            files.append(os.path.join(theme, "gtk-2.0", "main.rc"))
            files.append(os.path.join(theme, "gtk-2.0", "panel.rc"))
            files.append(os.path.join(theme, "gtk-2.0", "apps.rc"))
            files.append(os.path.join(theme, "gtk-2.0", "menubar-toolbar", "gtkrc"))
            files.append(os.path.join(theme, "metacity-1", "metacity-theme-2.xml"))
            files.append(os.path.join(theme, "metacity-1", "metacity-theme-3.xml"))
            files.append(os.path.join(theme, "libadwaita-1.5", "defaults-light.css"))
            files.append(os.path.join(theme, "libadwaita-1.5", "defaults-dark.css"))
            for file in files:
                if os.path.exists(file):
                    for accent in Y_HEX_ACCENT1:
                        os.system("sed -i s'/%(accent)s/%(color_accent)s/gI' %(file)s" % {'accent': accent, 'color_accent': y_hex_colors1[color], 'file': file})
                    for accent in Y_HEX_ACCENT2:
                        os.system("sed -i s'/%(accent)s/%(color_accent)s/gI' %(file)s" % {'accent': accent, 'color_accent': y_hex_colors2[color], 'file': file})
                    for accent in Y_HEX_ACCENT3:
                        os.system("sed -i s'/%(accent)s/%(color_accent)s/gI' %(file)s" % {'accent': accent, 'color_accent': y_hex_colors3[color], 'file': file})
                    for accent in Y_HEX_ACCENT4:
                        os.system("sed -i s'/%(accent)s/%(color_accent)s/gI' %(file)s" % {'accent': accent, 'color_accent': y_hex_colors4[color], 'file': file})

            # Remove metacity-theme-3.xml (it doesn't need to be derived since it's using GTK colors, and Cinnamon doesn't want to list it)
            os.system("rm -f %s" % os.path.join(theme, "metacity-1", "metacity-theme-3.xml"))

            directories = []
            directories.append(os.path.join(theme, "cinnamon/common-assets"))
            directories.append(os.path.join(theme, "cinnamon/light-assets"))
            directories.append(os.path.join(theme, "cinnamon/dark-assets"))
            for directory in directories:
                if os.path.exists(directory):
                    y_colorize_directory(directory, color)

            # Assets
            os.system("rm -rf %s/gtk-3.0/assets" % theme)
            os.system("rm -rf %s/gtk-2.0/assets" % theme)
            if variant == "-Dark":
                os.system("cp -R %s/gtk-2.0/assets-dark %s/gtk-2.0/assets" % (path, theme))
            else:
                os.system("cp -R %s/gtk-2.0/assets %s/gtk-2.0/assets" % (path, theme))
            os.system("cp -R %s/gtk-3.0/assets %s/gtk-3.0/assets" % (path, theme))


# Files
os.system("cp -R files/* ./")

# Fixes for accent-color

# The brown is too orange so libAdwaita uses orange instead of sand.
os.system(f"sed -i '/define-color accent_color/ s/aa876a/c8ac69/' usr/share/themes/Mint-L-Brown/gtk-3.0/gtk.css")
os.system(f"sed -i '/define-color accent_color/ s/aa876a/c8ac69/' usr/share/themes/Mint-L-Dark-Brown/gtk-3.0/gtk.css")
os.system(f"sed -i '/define-color accent_color/ s/aa876a/c8ac69/' usr/share/themes/Mint-L-Dark-Brown/gtk-3.0/gtk-dark.css")



#!/usr/bin/python3

# Mint-L
Y_HEX_ACCENT1 = ["#92b372"]  # BASE
Y_HEX_ACCENT2 = ["#8fa876"]  # DARK
Y_HEX_ACCENT3 = ["#afca95"]  # PRELIGHT/HOVER
Y_HEX_ACCENT4 = ["#779559"]  # PRESSED

y_hex_colors1 = {}
y_hex_colors2 = {}
y_hex_colors3 = {}
y_hex_colors4 = {}

y_hex_colors1["Aqua"] = "#6cabcd"
y_hex_colors2["Aqua"] = "#6aa0bd"
y_hex_colors3["Aqua"] = "#82b3ce"
y_hex_colors4["Aqua"] = "#52819a"

y_hex_colors1["Blue"] = "#5b73c4"
y_hex_colors2["Blue"] = "#596eb5"
y_hex_colors3["Blue"] = "#6e82c6"
y_hex_colors4["Blue"] = "#4b5a8d"

y_hex_colors1["Brown"] = "#aa876a"
y_hex_colors2["Brown"] = "#9c7e65"
y_hex_colors3["Brown"] = "#ad9078"
y_hex_colors4["Brown"] = "#796555"

y_hex_colors1["Grey"] = "#9d9d9d"
y_hex_colors2["Grey"] = "#8f8f8f"
y_hex_colors3["Grey"] = "#a3a3a3"
y_hex_colors4["Grey"] = "#727272"

y_hex_colors1["Orange"] = "#db9d61"
y_hex_colors2["Orange"] = "#cc9560"
y_hex_colors3["Orange"] = "#dcaa7a"
y_hex_colors4["Orange"] = "#a97747"

y_hex_colors1["Pink"] = "#c76199"
y_hex_colors2["Pink"] = "#b85f90"
y_hex_colors3["Pink"] = "#c975a3"
y_hex_colors4["Pink"] = "#924d73"

y_hex_colors1["Purple"] = "#8c6ec9"
y_hex_colors2["Purple"] = "#866cba"
y_hex_colors3["Purple"] = "#9b84cb"
y_hex_colors4["Purple"] = "#6a5497"

y_hex_colors1["Red"] = "#c15b58"
y_hex_colors2["Red"] = "#b35a57"
y_hex_colors3["Red"] = "#d1716f"
y_hex_colors4["Red"] = "#9a4a47"

y_hex_colors1["Sand"] = "#c8ac69"
y_hex_colors2["Sand"] = "#b89f65"
y_hex_colors3["Sand"] = "#c9b27c"
y_hex_colors4["Sand"] = "#947f51"

y_hex_colors1["Teal"] = "#5aaa9a"
y_hex_colors2["Teal"] = "#579c8e"
y_hex_colors3["Teal"] = "#6dbbab"
y_hex_colors4["Teal"] = "#4e8378"



#!/usr/bin/python3
import sys
import os
import re
import colorsys
HEX_COLOR_REGEX = r'#([A-Fa-f0-9]{6})'
RGB_COLOR_REGEX = r'rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)'

# Greens are between these hue values:
MIN_HUE = 57
MAX_HUE = 180

hex_colors = {}
rgb_colors = {}

IGNORED_COLORS = []
IGNORED_COLORS.append("73d216") # success color (always green)
IGNORED_COLORS.append("6db442") # suggested color (always green)

def hex_to_rgb(color):
    return tuple(int(color[i:i+2], 16) for i in (0, 2 ,4))

def rgb_to_hsv(color):
    red = int(color[0])
    green = int(color[1])
    blue = int(color[2])
    hue, saturation, value = colorsys.rgb_to_hsv(red/255., green/255., blue/255.)
    return (round(hue*360), round(saturation*100), round(value*100))

def parse_file(path):
    with open(path) as file:
        for color in re.findall(RGB_COLOR_REGEX, file.read()):
            if color not in rgb_colors.keys():
                rgb_colors[color] = 1
            else:
                rgb_colors[color] += 1
    with open(path) as file:
        for color in re.findall(HEX_COLOR_REGEX, file.read()):
            color = color.lower()
            if color not in IGNORED_COLORS:
                if color not in hex_colors.keys():
                    hex_colors[color] = 1
                else:
                    hex_colors[color] += 1

def parse_dir(path):
    for root,d_names,f_names in os.walk(path):
        for f in f_names:
            p = os.path.join(root, f)
            name, extension = os.path.splitext(p)
            if extension in [".css", ".scss", ".xml", ".svg", ".rc", ".xpm"]:
                parse_file(p)


if len(sys.argv) < 2:

    # Cinnamon
    parse_dir("src/Mint-L/cinnamon/sass")
    parse_dir("src/Mint-L/cinnamon/common-assets")
    parse_dir("src/Mint-L/cinnamon/dark-assets")
    parse_dir("src/Mint-L/cinnamon/light-assets")

    # GTK2
    parse_dir("src/Mint-L/gtk-2.0/menubar-toolbar")
    parse_file("src/Mint-L/gtk-2.0/apps.rc")
    parse_file("src/Mint-L/gtk-2.0/gtkrc")
    parse_file("src/Mint-L/gtk-2.0/gtkrc-dark")
    parse_file("src/Mint-L/gtk-2.0/gtkrc-darker")
    parse_file("src/Mint-L/gtk-2.0/main.rc")
    parse_file("src/Mint-L/gtk-2.0/panel.rc")
    parse_file("src/Mint-L/gtk-2.0/assets.svg")
    parse_file("src/Mint-L/gtk-2.0/assets-dark.svg")

    # GTK3
    parse_dir("src/Mint-L/gtk-3.0/sass")
    parse_file("src/Mint-L/gtk-3.0/assets.svg")

    # Metacity
    parse_dir("src/Mint-L/metacity-1")

    # Xfwm4 (ignore this one... loads of very similar colors in there)
    # parse_dir("src/Mint-L/xfwm4")

else:
    # parse a different directory
    parse_dir(sys.argv[1])

for color in rgb_colors.keys():
    occurences = rgb_colors[color]
    hue, saturation, value = rgb_to_hsv(color)
    if hue > MIN_HUE and hue < MAX_HUE:
        print (color, hue, saturation, value, "(appears %d times)" % occurences)

for color in sorted(hex_colors.keys()):
    occurences = hex_colors[color]
    rgb = hex_to_rgb(color)
    hue, saturation, value = rgb_to_hsv(rgb)
    if hue > MIN_HUE and hue < MAX_HUE:
        print (color, rgb, hue, saturation, value, "(appears %d times)" % occurences)

